/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.amap.lidar.amapvox.voxelisation.output;

import fr.amap.commons.raster.multiband.BCommon;
import fr.amap.commons.raster.multiband.BHeader;
import fr.amap.commons.raster.multiband.BSQ;
import fr.amap.lidar.amapvox.commons.Voxel;
import fr.amap.lidar.amapvox.voxelisation.Voxelization;
import java.io.File;
import java.io.IOException;
import java.math.BigInteger;
import javax.vecmath.Point3d;
import javax.vecmath.Point3i;
import org.apache.commons.lang3.ArrayUtils;

/**
 *
 * @author pverley
 */
public class LAIRasterOutput extends AbstractOutput {

    public LAIRasterOutput(Voxelization voxelisation) {
        super(voxelisation);
    }

    @Override
    public File[] write() throws IOException {

        File outputFile = voxCfg.getOutputFile();
        String fileName = outputFile.getName();
        float scale = 1.0f;
        float resolution = voxParam.infos.getResolution();
        int rasterXSize = (int) (Math.ceil(voxParam.infos.getSplit().x / scale));
        int rasterYSize = (int) (Math.ceil(voxParam.infos.getSplit().y / scale));
        BHeader header = new BHeader(rasterXSize, rasterYSize, 1, BCommon.NumberOfBits.N_BITS_32);
        header.setUlxmap(voxParam.infos.getMinCorner().x + (resolution / 2.0f));
        header.setUlymap(voxParam.infos.getMaxCorner().y - (resolution / 2.0f));
        header.setXdim((float) voxParam.infos.getVoxelSize().x);
        header.setYdim((float) voxParam.infos.getVoxelSize().y);
        BSQ raster = new BSQ(voxCfg.getOutputFile(), header);
        long valMax = 4294967294L;
        long valNoData = 4294967295L;
        double laiMax = 50000;
        header.addMetadata("NO_DATA", String.valueOf(valNoData));
        header.addMetadata("MAX_VAL", String.valueOf(valMax));
        header.addMetadata("MAX_VEG_M2", String.valueOf(laiMax));
        int nvox = voxParam.infos.getSplit().x * voxParam.infos.getSplit().y;
        int ivox = 1;
        LOGGER.info("writing file " + outputFile.getAbsolutePath());
        
        for (int i = 0; i < voxParam.infos.getSplit().x; i++) {
            for (int j = 0; j < voxParam.infos.getSplit().y; j++) {
                
                progress("writing file " + fileName, ivox++, nvox);

                double laiSum = 0;

                for (int k = voxParam.infos.getSplit().z - 1; k >= 0; k--) {

                    Voxel vox = voxelisation.getVoxel(i, j, k);

                    if (vox != null && vox.ground_distance >= 0 && vox.PadBVTotal > 0 && !Double.isNaN(vox.PadBVTotal)) {

                        //calcul de la position de départ
                        Point3d voxelPosition = voxelisation.getPosition(new Point3i(i, j, k));

                        Point3d subVoxelSize = new Point3d(voxParam.infos.getVoxelSize().x / voxParam.infos.getResolution(),
                                voxParam.infos.getVoxelSize().y / voxParam.infos.getResolution(),
                                voxParam.infos.getVoxelSize().z / voxParam.infos.getResolution());

                        Point3d startPosition = new Point3d(voxelPosition.x - (voxParam.infos.getVoxelSize().x - subVoxelSize.x) / 2.0,
                                voxelPosition.y - (voxParam.infos.getVoxelSize().y - subVoxelSize.y) / 2.0,
                                voxelPosition.z - (voxParam.infos.getVoxelSize().z - subVoxelSize.z) / 2.0);

                        Point3i nbSubVoxels = new Point3i((int) Math.ceil(voxParam.infos.getVoxelSize().x),
                                (int) Math.ceil(voxParam.infos.getVoxelSize().y),
                                (int) Math.ceil(voxParam.infos.getVoxelSize().z));

                        //parcours des sous-voxels
                        int nbSubVoxelsAboveGround = 0;

                        for (int i2 = 0; i2 < nbSubVoxels.x; i2++) {
                            for (int j2 = 0; j2 < nbSubVoxels.y; j2++) {
                                for (int k2 = nbSubVoxels.z - 1; k2 >= 0; k2--) {

                                    float groundDistance = voxelisation.getGroundDistance((float) (startPosition.x + (i2 * subVoxelSize.x)),
                                            (float) (startPosition.y + (j2 * subVoxelSize.y)),
                                            (float) (startPosition.z + (k2 * subVoxelSize.z)));

                                    if (groundDistance > 0) {
                                        nbSubVoxelsAboveGround++;
                                    }
                                }
                            }
                        }

                        float volume = (float) (subVoxelSize.x * subVoxelSize.y * subVoxelSize.z * nbSubVoxelsAboveGround);
                        double lai = vox.PadBVTotal * volume;

                        laiSum += lai;
                    }
                }

                long value = (long) ((laiSum / laiMax) * (valMax));

                String binaryString = Long.toBinaryString(value);
                byte[] bval = new BigInteger(binaryString, 2).toByteArray();
                ArrayUtils.reverse(bval);
                byte b0 = 0x0, b1 = 0x0, b2 = 0x0, b3 = 0x0;
                if (bval.length > 0) {
                    b0 = bval[0];
                }
                if (bval.length > 1) {
                    b1 = bval[1];
                }
                if (bval.length > 2) {
                    b2 = bval[2];
                }
                if (bval.length > 3) {
                    b3 = bval[3];
                }

                try {
                    raster.setPixel(i, voxParam.infos.getSplit().y - j - 1, 0, b0, b1, b2, b3);
                } catch (Exception ex) {
                    throw new IOException("Failed to write LAI pixel " + i + " " + j + " in file " + outputFile.getAbsolutePath(), ex);
                }
            }
        }

        File headerFile = raster.writeHeader();
        File bsqFile = raster.writeImage();

        return new File[] {headerFile, bsqFile};
    }

}
