/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.amap.commons.util.swing;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;

/**
 * Swing component to create content panel with a great alignment.
 */
public class JVerticalPanel extends JPanel {
    
    private final JPanel verticalPanel;

    public JVerticalPanel() {
        super(new BorderLayout());
        this.verticalPanel = new JPanel();
        verticalPanel.setLayout(new BoxLayout(verticalPanel, BoxLayout.Y_AXIS));
        this.add(verticalPanel, BorderLayout.PAGE_START);
    }
    
    public final void addLine(final JComponent leftContent, 
            final JComponent centerContent, final JComponent rightContent) {
        final JPanel linePanel = new JPanel();
        linePanel.setLayout(new BorderLayout());
        verticalPanel.add(linePanel);
        
        final JPanel leftPanel = new JPanel(new CardLayout(5, 5));
        if (leftContent != null) {
            leftPanel.add(leftContent);
        }
        linePanel.add(leftPanel, BorderLayout.LINE_START);
        
        final JPanel centerPanel = new JPanel(new CardLayout(5, 5));
        if (centerContent != null) {
            centerPanel.add(centerContent);
        }
        linePanel.add(centerPanel, BorderLayout.CENTER);
        
        final JPanel rightPanel = new JPanel(new CardLayout(5, 5));
        if (rightContent != null) {
            rightPanel.add(rightContent);
        }
        linePanel.add(rightPanel, BorderLayout.LINE_END);
    }

    public final void addLine(final JComponent leftContent, final JComponent rightContent) {
        final JPanel linePanel = new JPanel(new GridLayout(0, 2));
        verticalPanel.add(linePanel);
        
        final JPanel leftPanel = new JPanel(new CardLayout(5, 5));
        if (leftContent != null) {
            leftPanel.add(leftContent);
        }
        linePanel.add(leftPanel);
        
        final JPanel rightPanel = new JPanel(new CardLayout(5, 5));
        if (rightContent != null) {
            rightPanel.add(rightContent);
        }
        linePanel.add(rightPanel);
    }
    
    public final void addLine(final JComponent content) {
        final JPanel linePanel = new JPanel(new CardLayout(5, 5));
        verticalPanel.add(linePanel);
        
        if (content != null) {
            linePanel.add(content);
        }
    }
    
    public final void removeLine(final JComponent leftContent, 
            final JComponent centerContent, final JComponent rightContent) {
        for (Component container : verticalPanel.getComponents()) {
            if (container instanceof Container) {
                for (Component subContainer : ((Container) container).getComponents()) {
                    if (subContainer instanceof Container) {
                        for (Component component : ((Container) subContainer).getComponents()) {
                            if (component == leftContent 
                                    || component == rightContent 
                                    || component == centerContent) {
                                verticalPanel.remove(container);
                                return;
                            }
                        }
                    }
                }
            }
        }
    }
    
    public final void removeLine(final JComponent leftContent, final JComponent rightContent) {
        for (Component container : verticalPanel.getComponents()) {
            if (container instanceof Container) {
                for (Component subContainer : ((Container) container).getComponents()) {
                    if (subContainer instanceof Container) {
                        for (Component component : ((Container) subContainer).getComponents()) {
                            if (component == leftContent || component == rightContent) {
                                verticalPanel.remove(container);
                                return;
                            }
                        }
                    }
                }
            }
        }
    }
    
    public final void removeLine(final JComponent content) {
        for (Component container : verticalPanel.getComponents()) {
            if (container instanceof Container) {
                for (Component component : ((Container) container).getComponents()) {
                    if (component == content) {
                        verticalPanel.remove(container);
                        return;
                    }
                }
            }
        }
    }
    
}
